@@ -4,6 +4,7 @@ import platform
 import select
 import socket
 import ssl
+import threading
 import typing
 from unittest import mock
 
@@ -14,9 +15,6 @@ from dummyserver.testcase import SocketDummyServerTestCase, consume_socket
 from urllib3.util import ssl_
 from urllib3.util.ssltransport import SSLTransport
 
-if typing.TYPE_CHECKING:
-    from typing import Literal
-
 # consume_socket can iterate forever, we add timeouts to prevent halting.
 PER_TEST_TIMEOUT = 60
 
@@ -34,13 +32,11 @@ def server_client_ssl_contexts() -> tuple[ssl.SSLContext, ssl.SSLContext]:
 
 
 @typing.overload
-def sample_request(binary: Literal[True] = ...) -> bytes:
-    ...
+def sample_request(binary: typing.Literal[True] = ...) -> bytes: ...
 
 
 @typing.overload
-def sample_request(binary: Literal[False]) -> str:
-    ...
+def sample_request(binary: typing.Literal[False]) -> str: ...
 
 
 def sample_request(binary: bool = True) -> bytes | str:
@@ -54,7 +50,7 @@ def sample_request(binary: bool = True) -> bytes | str:
 
 
 def validate_request(
-    provided_request: bytearray, binary: Literal[False, True] = True
+    provided_request: bytearray, binary: typing.Literal[False, True] = True
 ) -> None:
     assert provided_request is not None
     expected_request = sample_request(binary)
@@ -62,18 +58,15 @@ def validate_request(
 
 
 @typing.overload
-def sample_response(binary: Literal[True] = ...) -> bytes:
-    ...
+def sample_response(binary: typing.Literal[True] = ...) -> bytes: ...
 
 
 @typing.overload
-def sample_response(binary: Literal[False]) -> str:
-    ...
+def sample_response(binary: typing.Literal[False]) -> str: ...
 
 
 @typing.overload
-def sample_response(binary: bool = ...) -> bytes | str:
-    ...
+def sample_response(binary: bool = ...) -> bytes | str: ...
 
 
 def sample_response(binary: bool = True) -> bytes | str:
@@ -111,20 +104,29 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
         cls.server_context, cls.client_context = server_client_ssl_contexts()
 
     def start_dummy_server(
-        self, handler: typing.Callable[[socket.socket], None] | None = None
+        self,
+        handler: typing.Callable[[socket.socket], None] | None = None,
+        validate: bool = True,
     ) -> None:
+        quit_event = threading.Event()
+
         def socket_handler(listener: socket.socket) -> None:
             sock = listener.accept()[0]
             try:
                 with self.server_context.wrap_socket(sock, server_side=True) as ssock:
-                    request = consume_socket(ssock)
+                    request = consume_socket(
+                        ssock,
+                        quit_event=quit_event,
+                    )
+                    if not validate:
+                        return
                     validate_request(request)
                     ssock.send(sample_response())
-            except (ConnectionAbortedError, ConnectionResetError):
+            except (ConnectionAbortedError, ConnectionResetError, BrokenPipeError):
                 return
 
         chosen_handler = handler if handler else socket_handler
-        self._start_server(chosen_handler)
+        self._start_server(chosen_handler, quit_event=quit_event)
 
     @pytest.mark.timeout(PER_TEST_TIMEOUT)
     def test_start_closed_socket(self) -> None:
@@ -138,7 +140,7 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
     @pytest.mark.timeout(PER_TEST_TIMEOUT)
     def test_close_after_handshake(self) -> None:
         """Socket errors should be bubbled up"""
-        self.start_dummy_server()
+        self.start_dummy_server(validate=False)
 
         sock = socket.create_connection((self.host, self.port))
         with SSLTransport(
@@ -184,9 +186,10 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
         """
 
         def shutdown_handler(listener: socket.socket) -> None:
-            with listener.accept()[0] as sock, self.server_context.wrap_socket(
-                sock, server_side=True
-            ) as ssl_sock:
+            with (
+                listener.accept()[0] as sock,
+                self.server_context.wrap_socket(sock, server_side=True) as ssl_sock,
+            ):
                 request = consume_socket(ssl_sock)
                 validate_request(request)
                 ssl_sock.sendall(sample_response())
@@ -223,9 +226,8 @@ class SingleTLSLayerTestCase(SocketDummyServerTestCase):
             cipher = ssock.cipher()
             assert type(cipher) is tuple
 
-            # No chosen protocol through ALPN or NPN.
+            # No chosen protocol.
             assert ssock.selected_alpn_protocol() is None
-            assert ssock.selected_npn_protocol() is None
 
             shared_ciphers = ssock.shared_ciphers()
             # SSLContext.shared_ciphers() changed behavior completely in a patch version.
@@ -442,7 +444,7 @@ class TlsInTlsTestCase(SocketDummyServerTestCase):
                 proxy_sock, self.client_context, server_hostname="localhost"
             ) as destination_sock:
                 file = destination_sock.makefile("rwb", buffering)
-                file.write(sample_request())  # type: ignore[call-overload]
+                file.write(sample_request())  # type: ignore[arg-type]
                 file.flush()
 
                 response = bytearray(65536)
