@@ -4,6 +4,7 @@ from __future__ import annotations
 
 import contextlib
 import errno
+import http.client
 import io
 import os
 import os.path
@@ -12,6 +13,8 @@ import shutil
 import socket
 import ssl
 import tempfile
+import threading
+import time
 import typing
 import zlib
 from collections import OrderedDict
@@ -19,6 +22,7 @@ from pathlib import Path
 from test import LONG_TIMEOUT, SHORT_TIMEOUT, notWindows, resolvesLocalhostFQDN
 from threading import Event
 from unittest import mock
+from urllib.parse import urlparse
 
 import pytest
 import trustme
@@ -30,7 +34,13 @@ from dummyserver.socketserver import (
     get_unreachable_address,
 )
 from dummyserver.testcase import SocketDummyServerTestCase, consume_socket
-from urllib3 import HTTPConnectionPool, HTTPSConnectionPool, ProxyManager, util
+from urllib3 import (
+    BaseHTTPResponse,
+    HTTPConnectionPool,
+    HTTPSConnectionPool,
+    ProxyManager,
+    util,
+)
 from urllib3._collections import HTTPHeaderDict
 from urllib3.connection import HTTPConnection, _get_default_user_agent
 from urllib3.connectionpool import _url_from_pool
@@ -47,7 +57,7 @@ from urllib3.util import ssl_, ssl_wrap_socket
 from urllib3.util.retry import Retry
 from urllib3.util.timeout import Timeout
 
-from .. import LogRecorder, has_alpn
+from .. import LogRecorder
 
 if typing.TYPE_CHECKING:
     from _typeshed import StrOrBytesPath
@@ -106,9 +116,6 @@ class TestSNI(SocketDummyServerTestCase):
 
 class TestALPN(SocketDummyServerTestCase):
     def test_alpn_protocol_in_first_request_packet(self) -> None:
-        if not has_alpn():
-            pytest.skip("ALPN-support not available")
-
         done_receiving = Event()
         self.buf = b""
 
@@ -955,27 +962,34 @@ class TestSocketClosing(SocketDummyServerTestCase):
             assert response.connection is None
 
     def test_socket_close_socket_then_file(self) -> None:
-        def consume_ssl_socket(listener: socket.socket) -> None:
+        quit_event = threading.Event()
+
+        def consume_ssl_socket(
+            listener: socket.socket,
+        ) -> None:
             try:
-                with listener.accept()[0] as sock, original_ssl_wrap_socket(
-                    sock,
-                    server_side=True,
-                    keyfile=DEFAULT_CERTS["keyfile"],
-                    certfile=DEFAULT_CERTS["certfile"],
-                    ca_certs=DEFAULT_CA,
-                ) as ssl_sock:
-                    consume_socket(ssl_sock)
+                with (
+                    listener.accept()[0] as sock,
+                    original_ssl_wrap_socket(
+                        sock,
+                        server_side=True,
+                        keyfile=DEFAULT_CERTS["keyfile"],
+                        certfile=DEFAULT_CERTS["certfile"],
+                        ca_certs=DEFAULT_CA,
+                    ) as ssl_sock,
+                ):
+                    consume_socket(ssl_sock, quit_event=quit_event)
             except (ConnectionResetError, ConnectionAbortedError, OSError):
                 pass
 
-        self._start_server(consume_ssl_socket)
-        with socket.create_connection(
-            (self.host, self.port)
-        ) as sock, contextlib.closing(
-            ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
-        ) as ssl_sock, ssl_sock.makefile(
-            "rb"
-        ) as f:
+        self._start_server(consume_ssl_socket, quit_event=quit_event)
+        with (
+            socket.create_connection((self.host, self.port)) as sock,
+            contextlib.closing(
+                ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
+            ) as ssl_sock,
+            ssl_sock.makefile("rb") as f,
+        ):
             ssl_sock.close()
             f.close()
             with pytest.raises(OSError):
@@ -983,32 +997,98 @@ class TestSocketClosing(SocketDummyServerTestCase):
             assert ssl_sock.fileno() == -1
 
     def test_socket_close_stays_open_with_makefile_open(self) -> None:
+        quit_event = threading.Event()
+
         def consume_ssl_socket(listener: socket.socket) -> None:
             try:
-                with listener.accept()[0] as sock, original_ssl_wrap_socket(
-                    sock,
-                    server_side=True,
-                    keyfile=DEFAULT_CERTS["keyfile"],
-                    certfile=DEFAULT_CERTS["certfile"],
-                    ca_certs=DEFAULT_CA,
-                ) as ssl_sock:
-                    consume_socket(ssl_sock)
+                with (
+                    listener.accept()[0] as sock,
+                    original_ssl_wrap_socket(
+                        sock,
+                        server_side=True,
+                        keyfile=DEFAULT_CERTS["keyfile"],
+                        certfile=DEFAULT_CERTS["certfile"],
+                        ca_certs=DEFAULT_CA,
+                    ) as ssl_sock,
+                ):
+                    consume_socket(ssl_sock, quit_event=quit_event)
             except (ConnectionResetError, ConnectionAbortedError, OSError):
                 pass
 
-        self._start_server(consume_ssl_socket)
-        with socket.create_connection(
-            (self.host, self.port)
-        ) as sock, contextlib.closing(
-            ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
-        ) as ssl_sock, ssl_sock.makefile(
-            "rb"
+        self._start_server(consume_ssl_socket, quit_event=quit_event)
+        with (
+            socket.create_connection((self.host, self.port)) as sock,
+            contextlib.closing(
+                ssl_wrap_socket(sock, server_hostname=self.host, ca_certs=DEFAULT_CA)
+            ) as ssl_sock,
+            ssl_sock.makefile("rb"),
         ):
             ssl_sock.close()
             ssl_sock.close()
             ssl_sock.sendall(b"hello")
             assert ssl_sock.fileno() > 0
 
+    def test_socket_shutdown_stops_recv(self) -> None:
+        timed_out, starting_read = Event(), Event()
+
+        def socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+
+            ssl_sock = original_ssl_wrap_socket(
+                sock,
+                server_side=True,
+                keyfile=DEFAULT_CERTS["keyfile"],
+                certfile=DEFAULT_CERTS["certfile"],
+                ca_certs=DEFAULT_CA,
+            )
+
+            # Consume request
+            buf = b""
+            while not buf.endswith(b"\r\n\r\n"):
+                buf = ssl_sock.recv(65535)
+
+            # Send incomplete message (note Content-Length)
+            ssl_sock.send(
+                b"HTTP/1.1 200 OK\r\n"
+                b"Content-Type: text/plain\r\n"
+                b"Content-Length: 10\r\n"
+                b"\r\n"
+                b"Hi-"
+            )
+            timed_out.wait(5)
+            ssl_sock.close()
+
+        self._start_server(socket_handler)
+
+        class TestClient(threading.Thread):
+            def __init__(self, host: str, port: int) -> None:
+                super().__init__()
+                self.host, self.port = host, port
+                self.response: BaseHTTPResponse | None = None
+
+            def run(self) -> None:
+                with HTTPSConnectionPool(
+                    self.host, self.port, ca_certs=DEFAULT_CA
+                ) as pool:
+                    self.response = pool.urlopen(
+                        "GET", "/", preload_content=False, retries=0
+                    )
+                    with pytest.raises(ProtocolError, match="Connection broken"):
+                        starting_read.set()
+                        self.response.read()
+
+        test_client = TestClient(self.host, self.port)
+        test_client.start()
+        # First, wait to make sure the client is really stuck reading
+        starting_read.wait(5)
+        time.sleep(LONG_TIMEOUT)
+        # Calling shutdown here calls shutdown() on the underlying socket,
+        # so that the remaining read will fail instead of blocking
+        # indefinitely
+        assert test_client.response is not None
+        test_client.response.shutdown()
+        timed_out.set()
+
 
 class TestProxyManager(SocketDummyServerTestCase):
     def test_simple(self) -> None:
@@ -1133,6 +1213,27 @@ class TestProxyManager(SocketDummyServerTestCase):
                     retries=False,
                 )
 
+    def test_tunnel_sets_http_11_alpn(self) -> None:
+        done_receiving = Event()
+        self.buf = b""
+
+        def socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+
+            self.buf = sock.recv(65536)  # We only accept one packet
+            done_receiving.set()  # let the test know it can proceed
+            sock.close()
+
+        self._start_server(socket_handler)
+        base_url = f"https://{self.host}:{self.port}"
+        with proxy_from_url(base_url) as proxy:
+            with pytest.raises(MaxRetryError):
+                proxy.request("GET", "https://localhost/")
+
+        done_receiving.wait()
+        assert b"http/1.1" in self.buf
+        assert b"h2" not in self.buf
+
     def test_connect_reconn(self) -> None:
         def proxy_ssl_one(listener: socket.socket) -> None:
             sock = listener.accept()[0]
@@ -1189,7 +1290,7 @@ class TestProxyManager(SocketDummyServerTestCase):
             r = conn.urlopen("GET", url, retries=0)
             assert r.status == 200
 
-    def test_connect_ipv6_addr(self) -> None:
+    def test_connect_ipv6_addr_from_host(self) -> None:
         ipv6_addr = "2001:4998:c:a06::2:4008"
 
         def echo_socket_handler(listener: socket.socket) -> None:
@@ -1229,13 +1330,75 @@ class TestProxyManager(SocketDummyServerTestCase):
 
         with proxy_from_url(base_url, cert_reqs="NONE") as proxy:
             url = f"https://[{ipv6_addr}]"
+
+            # Try with connection_from_host
+            parsed_request_url = urlparse(url)
+
+            conn = proxy.connection_from_host(
+                scheme=parsed_request_url.scheme.lower(),
+                host=parsed_request_url.hostname,
+                port=parsed_request_url.port,
+            )
+            try:
+                with pytest.warns(InsecureRequestWarning):
+                    r = conn.urlopen("GET", url, retries=0)
+                assert r.status == 200
+            except MaxRetryError:
+                pytest.fail(
+                    "Invalid IPv6 format in HTTP CONNECT request when using connection_from_host"
+                )
+
+    def test_connect_ipv6_addr_from_url(self) -> None:
+        ipv6_addr = "2001:4998:c:a06::2:4008"
+
+        def echo_socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+
+            buf = b""
+            while not buf.endswith(b"\r\n\r\n"):
+                buf += sock.recv(65536)
+            s = buf.decode("utf-8")
+
+            if s.startswith(f"CONNECT [{ipv6_addr}]:443"):
+                sock.send(b"HTTP/1.1 200 Connection Established\r\n\r\n")
+                ssl_sock = original_ssl_wrap_socket(
+                    sock,
+                    server_side=True,
+                    keyfile=DEFAULT_CERTS["keyfile"],
+                    certfile=DEFAULT_CERTS["certfile"],
+                )
+                buf = b""
+                while not buf.endswith(b"\r\n\r\n"):
+                    buf += ssl_sock.recv(65536)
+
+                ssl_sock.send(
+                    b"HTTP/1.1 200 OK\r\n"
+                    b"Content-Type: text/plain\r\n"
+                    b"Content-Length: 2\r\n"
+                    b"Connection: close\r\n"
+                    b"\r\n"
+                    b"Hi"
+                )
+                ssl_sock.close()
+            else:
+                sock.close()
+
+        self._start_server(echo_socket_handler)
+        base_url = f"http://{self.host}:{self.port}"
+
+        with proxy_from_url(base_url, cert_reqs="NONE") as proxy:
+            url = f"https://[{ipv6_addr}]"
+
+            # Try with connection_from_url
             conn = proxy.connection_from_url(url)
             try:
                 with pytest.warns(InsecureRequestWarning):
                     r = conn.urlopen("GET", url, retries=0)
                 assert r.status == 200
             except MaxRetryError:
-                pytest.fail("Invalid IPv6 format in HTTP CONNECT request")
+                pytest.fail(
+                    "Invalid IPv6 format in HTTP CONNECT request when using connection_from_url"
+                )
 
     @pytest.mark.parametrize("target_scheme", ["http", "https"])
     def test_https_proxymanager_connected_to_http_proxy(
@@ -1263,6 +1426,84 @@ class TestProxyManager(SocketDummyServerTestCase):
                 e.value.reason
             )
 
+    def test_proxy_status_not_ok(self) -> None:
+        def http_socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+            consume_socket(sock)
+            sock.send(b"HTTP/1.0 501 Not Implemented\r\nConnection: close\r\n\r\n")
+            sock.close()
+
+        self._start_server(http_socket_handler)
+        base_url = f"http://{self.host}:{self.port}"
+
+        with ProxyManager(base_url) as proxy:
+            with pytest.raises(MaxRetryError) as e:
+                proxy.request("GET", "https://example.com", retries=0)
+
+            assert type(e.value.reason) is ProxyError
+            assert e.value.reason.args[0] == "Unable to connect to proxy"
+            assert type(e.value.reason.args[1]) is OSError
+            assert (
+                str(e.value.reason.args[1])
+                == "Tunnel connection failed: 501 Not Implemented"
+            )
+
+    def test_early_eof_doesnt_cause_infinite_loop(self) -> None:
+        def http_socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+            consume_socket(sock)
+            sock.send(b"HTTP/1.0 200 OK\r\n")
+            sock.close()
+
+        self._start_server(http_socket_handler)
+        base_url = f"http://{self.host}:{self.port}"
+
+        with ProxyManager(base_url) as proxy:
+            with pytest.raises(MaxRetryError):
+                proxy.request("GET", "https://example.com", retries=0)
+
+    def test_header_longer_than_maxline(self) -> None:
+        def http_socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+            consume_socket(sock)
+            sock.send(
+                b"HTTP/1.0 200 OK\r\nThis-Header-Is-Too-Long: Way-Too-Long\r\n\r\n"
+            )
+            sock.close()
+
+        self._start_server(http_socket_handler)
+        base_url = f"http://{self.host}:{self.port}"
+
+        with mock.patch("http.client._MAXLINE", 17):
+            with ProxyManager(base_url) as proxy:
+                with pytest.raises(MaxRetryError) as e:
+                    proxy.request("GET", "https://example.com", retries=0)
+
+            assert type(e.value.reason) is ProtocolError
+            assert e.value.reason.args[0] == "Connection aborted."
+            assert type(e.value.reason.args[1]) is http.client.LineTooLong
+            assert (
+                str(e.value.reason.args[1])
+                == "got more than 17 bytes when reading header line"
+            )
+
+    def test_debuglevel(self, capsys: pytest.CaptureFixture[str]) -> None:
+        def http_socket_handler(listener: socket.socket) -> None:
+            sock = listener.accept()[0]
+            consume_socket(sock)
+            sock.send(b"HTTP/1.0 200 OK\r\nExample-Header: Example-Value\r\n\r\n")
+            sock.close()
+
+        self._start_server(http_socket_handler)
+        base_url = f"http://{self.host}:{self.port}"
+
+        with mock.patch("http.client.HTTPConnection.debuglevel", 1):
+            with ProxyManager(base_url) as proxy:
+                with pytest.raises(MaxRetryError):
+                    proxy.request("GET", "https://example.com", retries=0)
+
+        assert "header: Example-Header: Example-Value\r\n\n" in capsys.readouterr().out
+
 
 class TestSSL(SocketDummyServerTestCase):
     def test_ssl_failure_midway_through_conn(self) -> None:
@@ -2232,11 +2473,28 @@ class TestBrokenPipe(SocketDummyServerTestCase):
 
 class TestMultipartResponse(SocketDummyServerTestCase):
     def test_multipart_assert_header_parsing_no_defects(self) -> None:
+        quit_event = threading.Event()
+
         def socket_handler(listener: socket.socket) -> None:
             for _ in range(2):
-                sock = listener.accept()[0]
-                while not sock.recv(65536).endswith(b"\r\n\r\n"):
-                    pass
+                listener.settimeout(LONG_TIMEOUT)
+
+                while True:
+                    if quit_event and quit_event.is_set():
+                        return
+                    try:
+                        sock = listener.accept()[0]
+                        break
+                    except (TimeoutError, socket.timeout):
+                        continue
+
+                sock.settimeout(LONG_TIMEOUT)
+                while True:
+                    if quit_event and quit_event.is_set():
+                        sock.close()
+                        return
+                    if sock.recv(65536).endswith(b"\r\n\r\n"):
+                        break
 
                 sock.sendall(
                     b"HTTP/1.1 404 Not Found\r\n"
@@ -2252,7 +2510,7 @@ class TestMultipartResponse(SocketDummyServerTestCase):
                 )
                 sock.close()
 
-        self._start_server(socket_handler)
+        self._start_server(socket_handler, quit_event=quit_event)
         from urllib3.connectionpool import log
 
         with mock.patch.object(log, "warning") as log_warning:
@@ -2308,15 +2566,26 @@ class TestContentFraming(SocketDummyServerTestCase):
     def test_chunked_specified(
         self, method: str, chunked: bool, body_type: str
     ) -> None:
+        quit_event = threading.Event()
         buffer = bytearray()
         expected_bytes = b"\r\n\r\na\r\nxxxxxxxxxx\r\n0\r\n\r\n"
 
         def socket_handler(listener: socket.socket) -> None:
             nonlocal buffer
-            sock = listener.accept()[0]
-            sock.settimeout(0)
+            listener.settimeout(LONG_TIMEOUT)
+            while True:
+                if quit_event.is_set():
+                    return
+                try:
+                    sock = listener.accept()[0]
+                    break
+                except (TimeoutError, socket.timeout):
+                    continue
+            sock.settimeout(LONG_TIMEOUT)
 
             while expected_bytes not in buffer:
+                if quit_event.is_set():
+                    return
                 with contextlib.suppress(BlockingIOError):
                     buffer += sock.recv(65536)
 
@@ -2327,12 +2596,12 @@ class TestContentFraming(SocketDummyServerTestCase):
             )
             sock.close()
 
-        self._start_server(socket_handler)
+        self._start_server(socket_handler, quit_event=quit_event)
 
         body: typing.Any
         if body_type == "generator":
 
-            def body_generator() -> typing.Generator[bytes, None, None]:
+            def body_generator() -> typing.Generator[bytes]:
                 yield b"x" * 10
 
             body = body_generator()
@@ -2370,7 +2639,7 @@ class TestContentFraming(SocketDummyServerTestCase):
 
         if body_type == "generator":
 
-            def body_generator() -> typing.Generator[bytes, None, None]:
+            def body_generator() -> typing.Generator[bytes]:
                 yield b"x" * 10
 
             body = body_generator()
